/*
  (c) Copyright 2004, 2005, 2006, 2007, 2008, 2009 Hewlett-Packard Development Company, LP, all rights reserved.
  (c) Copyright 2009 TopQuadrant, Inc.
  [See end of file]
  $Id: GraphEvents.java,v 1.4 2010/05/24 13:25:47 andy_seaborne Exp $
*/
package com.hp.hpl.jena.graph;

/**
    GraphEvents is the base class for Jena general graph events. Each graph event
    has a title and some content.
    
    @author kers
 */
public class GraphEvents
	{
	public static final GraphEvents removeAll = new GraphEvents( "removeAll", "" ){
    	@Override
		public Triple getTriple() { return Triple.ANY; }
      };
	
	public static final GraphEvents startRead = new GraphEvents( "startRead", "" );
	
	public static final GraphEvents finishRead = new GraphEvents( "finishRead", "" );
	
    final private String title;
    final protected Object content;
    
    public GraphEvents( String title, Object content )
        { this.title = title;
        this.content = content; }
    
    @Override
    public boolean equals( Object o )
        { return o instanceof GraphEvents && same( (GraphEvents) o ); }
    
    public boolean same( GraphEvents o )
        { return title.equals( o.title ) && content.equals( o.content ); }
   
	public static GraphEvents remove( Node s, Node p, Node o )
	    { return new GraphEvents( "remove", Triple.create( s, p, o ) ){
	    	@Override
			public Triple getTriple() { return (Triple)content; }
	      };
	    }
	
	@Override
    public String toString()
	    { return "<GE " + title + ">"; }

	/**
     * A fluid triple relevant to the event, or null if not applicable.
     * For events generated by the Jena code base the title is always a String,
     * and is one of:
     * <dl>
     * <dt><q>startRead</q></dt>
     * <dt><q>finishRead</q></dt><dd><code>null</code></dd>
     * <dt><q>removeAll</q></dt><dd>{@link com.hp.hpl.jena.graph.Triple#ANY}</dd>
     * <dt><q>remove</q></dt><dd>The fluid triple being removed.</dd>
     * </dl>
     * @return An interesting triple.
     */
    public Triple getTriple()
        { return null; }
	
	
	/**
     * For events generated by the Jena code base the content is one of:
     * <dl>
     * <dt><q>startRead</q></dt>
     * <dt><q>finishRead</q></dt>
     * <dt><q>removeAll</q></dt><dd>The empty string.</dd>
     * <dt><q>remove</q></dt><dd>The (non-concrete) triple being removed.</dd>
     * </dl>
     */
    public Object getContent()
        { return content; }
    

    /**
     * For events generated by the Jena code base the title is always a String,
     * and is one of:
     * <dl>
     * <dt><q>startRead</q></dt><dd>{@link #startRead}</dd>
     * <dt><q>finishRead</q></dt><dd>{@link #finishRead}</dd>
     * <dt><q>removeAll</q></dt><dd>{@link #removeAll}</dd>
     * <dt><q>remove</q></dt><dd>{@link #remove(Node, Node, Node)}</dd>
     * </dl>
     */
    public Object getTitle()
        { return title; }
	}

/*
(c) Copyright 2004, 2005, 2006, 2007, 2008, 2009 Hewlett-Packard Development Company, LP
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/